#version 330 core

in vec2 TexCoords;

out vec4 outColor;

uniform sampler2D screenTexture;

vec3 Uncharted2Tonemap(vec3 x)
{
	float A = 0.15;
	float B = 0.50;
	float C = 0.10;
	float D = 0.20;
	float E = 0.02;
	float F = 0.30;

	return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}


vec2 CRTCurveUV( vec2 uv )
{
    uv = uv * 2.0 - 1.0;
    vec2 offset = abs( uv.yx ) / vec2( 6.0, 4.0 );
    uv = uv + uv * offset * offset;
    uv = uv * 0.5 + 0.5;
    return uv;
}

void DrawVignette( inout vec3 color, vec2 uv )
{    
	//color = color * 1.2f;
    float vignette = uv.x * uv.y * ( 1.0 - uv.x ) * ( 1.0 - uv.y );
    vignette = clamp( pow( 16.0 * vignette, 0.4 ), 0.0, 1.0 );
    color *= vignette;
}

void DrawScanline( inout vec3 color, vec2 uv )
{
    float scanline 	= clamp( 0.85 + 0.15 * cos( 3.14 * ( uv.y + 0.008 * 20 ) * 240.0 * 0.9 ), 0.0, 1.0 );
    float grille 	= 0.85 + 0.15 * clamp( 1.5 * cos( 3.14 * uv.x * 640.0 * 1.0 ), 0.0, 1.0 );    
    color *= scanline * grille * 1.4;
}

void CRTColorWarmth(inout vec3 color)
{
	const vec3 sat = vec3(.0f,.00f,.0f);
	const vec3 offset = vec3(.05f,.02f,.0f);
	
	color = min(vec3(1.0,1.0,1.0),(color*(1+sat)) + offset); 
}

void CRTBorderBleed(inout vec3 color, vec2 uv )
{
const vec3 bleedColor = vec3 (.2f, 0.1f, 0.0f);
	float vignette = uv.x * uv.y * ( 1.0 - uv.x ) * ( 1.0 - uv.y );
    vignette = clamp( pow( 16.0 * vignette, 0.4 ), 0.0, 1.0 );
	color = min(vec3(1.0f,1.0f,1.0f), color + (bleedColor * (1-vignette)));
}


void CRTColorOffset(inout vec3 color) 
{
	const vec3 bleed = vec3(0.15f,0.1f,0.05f);
	const int linewidth = 2;

	vec3 pos = vec3(gl_FragCoord);
	pos = pos/linewidth;
	vec3 yOff = vec3(0.0,1.0,2.0);
	vec3 damp = vec3(min(1.0 , mod(pos.x + yOff.x,3)), min(1.0 , mod(pos.x + yOff.y ,3)), min(1.0 , mod(pos.x + yOff.z,3)));
	color = ((damp * (bleed - 1)) + 1) * color;
}


void main()
{
	const float gamma = 2.2;

	vec3 colorHDR = texture(screenTexture, TexCoords).rgb;
	// reinhard
	//vec3 color = colorHDR / (colorHDR + vec3(1.0)); 

	vec3 color = Uncharted2Tonemap(colorHDR);
	color = pow(color, vec3(1.0 / gamma));

	vec2 uv    = TexCoords;
    vec2 crtUV = CRTCurveUV( uv );
    if ( crtUV.x < 0.0 || crtUV.x > 1.0 || crtUV.y < 0.0 || crtUV.y > 1.0 )
    {
        color = vec3( 0.0, 0.0, 0.0 );
    }


	CRTColorOffset(color);

 
    DrawScanline( color, uv );

	CRTColorWarmth(color);

	
	CRTBorderBleed(color, uv);

	DrawVignette( color, crtUV );


	outColor = vec4(color, 1.0f);
}